/*
 	AirTagDB.cpp		AirTag信号強度リストクラス
 		Programmed by ハンダマスターかしま 2024/6/27
 		The MIT License (MIT)
 		Copyright (c) 2024 H.Kashima <kashima@kaele.com>
 		Kashi Electronics Designs
 */
#include "AirTagDB.h"

//
// タグリスト管理クラス
//
// コンストラクタ
AirTagDB::AirTagDB() {
	clear();
}

// 全リストを消去
void AirTagDB::clear(){
	memset(tags, 0, sizeof(tags));
	for (int i = 0; i < MAX_TAGS; i++)
		tags[i].elaps = -1;
}

// AirTag情報を追加・更新
bool AirTagDB::append(AirTagInfo tag) {
	// 同じMACアドレスであれば情報更新
	for (int i = 0; i < MAX_TAGS; i++) {
		if(tags[i].elaps >= 0 && tags[i].addr == tag.addr) {
			tags[i] = tag;
			return false;
		}
	}
	// 既存リストに無いので新しいMACアドレスとして追加
	for (int i = 0; i < MAX_TAGS; i++) {
		if (tags[i].elaps < 0 || tags[i].elaps < (tag.elaps - EXPIRE_TIME) ) {
			tags[i] = tag;
			return true;
		}
	}
	return false;
}

// 15分経ってMACアドレスが保証できなくなった情報を破棄、有効エントリ数を返す
int AirTagDB::expire(int elaps) {
	int c = 0;
	for (int i = 0; i < MAX_TAGS; i++) {
		if( tags[i].elaps < (elaps - EXPIRE_TIME) ) {
			tags[i].elaps = -1;

		} else if(tags[i].elaps >= 0)
			c++;
	}
	return c;
}

// 有効なエントリ数を返す
int AirTagDB::count(){
	int c = 0;
	for (int i = 0; i < MAX_TAGS; i++) {
		if( tags[i].elaps >= 0) c++;
	}
	return c;
}

// rssiがトップのポインターを返す
AirTagInfo* AirTagDB::top_rssi() {
	int top = -999;							// -999は欠損値(無効データ)に
	int i, j;

	for (i = 0; i < MAX_TAGS; i++) {
		if( tags[i].elaps >= 0 && tags[i].rssi > top){
			top = tags[i].rssi;
			j = i;
		}
	}
	if (top == -999)
		return NULL;						// トップ不在なら NULLを返却
//	else
//		Serial.printf("rssi: %d\n", top);
	return &tags[j];
}

// MACアドレスを探す
AirTagInfo* AirTagDB::findMAC(BLEAddress a) {
	for (int i = 0; i < MAX_TAGS; i++) {
		if( tags[i].elaps >= 0 && tags[i].addr == a)
			return &tags[i];
	}
	return NULL;
}

// rssiの値でソートする
int AirTagDB::sort_rssi(){
	qsort( tags, MAX_TAGS, sizeof(AirTagInfo), cmp_rssi);
	return count();
}

// qsort比較関数
int AirTagDB::cmp_rssi(const void *a, const void *b){
	if(((AirTagInfo *)a)->rssi > ((AirTagInfo *)b)->rssi)
		return -1;
	else
		return 1;
}
