/*
	AirTagDB.h			AirTag信号強度リストクラス
 		Programmed by ハンダマスターかしま 2024/6/27
		The MIT License (MIT)
 		Copyright (c) 2024 H.Kashima <kashima@kaele.com>
 		Kashi Electronics Designs
 */
#include <BLEAddress.h>

#ifndef __AIRTAGDB__
#define __AIRTAGDB__

#define MAX_TAGS 256						// 最大リスト数
#define EXPIRE_TIME 900						// MACアドレスの有効期限

//
// タグ情報クラス
//
class AirTagInfo {
	public:
		BLEAddress addr = BLEAddress((uint8_t *)"\0\0\0\0\0\0");	// Find My(AirTag) MACアドレス
		int elaps, rssi;					// 受信時刻, 受信レベル
		int8_t plen, stat;					// ペイロード長, ステータス
};

//
// タグリスト管理クラス
//
class AirTagDB {
public:
	AirTagInfo  tags[MAX_TAGS];				// データーテーブル

	AirTagDB();								// コンストラクタ
	void clear();							// 全リストを消去
	bool append(AirTagInfo tag);			// AirTag情報を追加・更新
	int expire(int elaps);					// 15分経ってMACアドレスが保証できなくなった情報を破棄
	int count();							// 有効なエントリ数を返す
	AirTagInfo* top_rssi();					// rssiがトップのポインターを返す
	AirTagInfo* findMAC(BLEAddress a);		// MACアドレスで探す
	int sort_rssi();						// rssiの値でソートする

private:
	static int cmp_rssi(const void *a, const void *b);		// qsort比較関数
};

#endif // __AIRTAGDB__
